@@ -194,7 +194,7 @@ class SharedState {
       : cv_(&mu_),
         cache_bench_(cache_bench) {}
 
-  ~SharedState() {}
+  ~SharedState() = default;
 
   port::Mutex* GetMutex() { return &mu_; }
 
@@ -425,7 +425,7 @@ class CacheBench {
     }
   }
 
-  ~CacheBench() {}
+  ~CacheBench() = default;
 
   void PopulateCache() {
     Random64 rnd(FLAGS_seed);
