@@ -12,6 +12,7 @@ package org.rocksdb;
  * number of disk seeks form a handful to a single disk seek per
  * DB::Get() call.
  */
+//TODO(AR) should be renamed FilterPolicy
 public abstract class Filter extends RocksObject {
 
   protected Filter(final long nativeHandle) {
@@ -31,5 +32,9 @@ public abstract class Filter extends RocksObject {
   }
 
   @Override
-  protected final native void disposeInternal(final long handle);
+  protected final void disposeInternal(final long handle) {
+    disposeInternalJni(handle);
+  }
+
+  private static native void disposeInternalJni(final long handle);
 }
