@@ -3,7 +3,6 @@
 //  COPYING file in the root directory) and Apache 2.0 License
 //  (found in the LICENSE.Apache file in the root directory).
 
-#ifndef ROCKSDB_LITE
 
 #pragma once
 
@@ -12,16 +11,16 @@
 #include "rocksdb/compaction_filter.h"
 #include "rocksdb/slice.h"
 
-namespace rocksdb {
+namespace ROCKSDB_NAMESPACE {
 
 class RemoveEmptyValueCompactionFilter : public CompactionFilter {
  public:
-    const char* Name() const override;
-    bool Filter(int level,
-        const Slice& key,
-        const Slice& existing_value,
-        std::string* new_value,
-        bool* value_changed) const override;
+  static const char* kClassName() { return "RemoveEmptyValueCompactionFilter"; }
+
+  const char* Name() const override { return kClassName(); }
+
+  bool Filter(int level, const Slice& key, const Slice& existing_value,
+              std::string* new_value, bool* value_changed) const override;
 };
-}  // namespace rocksdb
-#endif  // !ROCKSDB_LITE
+
+}  // namespace ROCKSDB_NAMESPACE
